@@ -7,6 +7,7 @@ from rasa.shared.constants import (
     DEFAULT_DOMAIN_PATH,
     DEFAULT_MODELS_PATH,
     DEFAULT_DATA_PATH,
+    DEFAULT_ENDPOINTS_PATH,
 )
 
 
@@ -90,8 +91,9 @@ def add_out_param(
 def add_endpoint_param(
     parser: Union[argparse.ArgumentParser, argparse._ActionsContainer],
     help_text: Text,
-    default: Optional[Text] = None,
+    default: Optional[Text] = DEFAULT_ENDPOINTS_PATH,
 ) -> None:
+    """Adds an option to an argument parser to configure endpoints path."""
     parser.add_argument("--endpoints", type=str, default=default, help=help_text)
 
 
